@@ -5,7 +5,7 @@ Zabbix::Check::Disk - Zabbix check for disk
 
 =head1 VERSION
 
-version 1.06
+version 1.09
 
 =head1 SYNOPSIS
 
@@ -60,7 +60,7 @@ BEGIN
 {
 	require Exporter;
 	# set the version for version checking
-	our $VERSION     = '1.06';
+	our $VERSION     = '1.09';
 	# Inherit from Exporter to export functions and variables
 	our @ISA         = qw(Exporter);
 	# Functions and variables which are exported by default
@@ -76,7 +76,7 @@ sub disks
 	for my $blockpath (glob("/sys/dev/block/*"))
 	{
 		next unless -f "$blockpath/uevent";
-		my $uevent = read_file("$blockpath/uevent");
+		my $uevent = read_file("$blockpath/uevent", { err_mode => "quiet" });
 		my ($major) = $uevent =~ /^\QMAJOR=\E(.*)/m;
 		my ($minor) = $uevent =~ /^\QMINOR=\E(.*)/m;
 		my ($devname) = $uevent =~ /^\QDEVNAME=\E(.*)/m;
@@ -89,20 +89,20 @@ sub disks
 			devpath => $devpath,
 			major => $major,
 			minor => $minor,
-			size => (-f "$blockpath/size" and $_ = read_file("$blockpath/size"))? int(s/^\s+|\s+$//gr)*512: undef,
-			removable => (-f "$blockpath/removable" and $_ = read_file("$blockpath/removable"))? s/^\s+|\s+$//gr: undef,
-			partition => (-f "$blockpath/partition" and $_ = read_file("$blockpath/partition"))? s/^\s+|\s+$//gr: undef,
+			size => (-f "$blockpath/size" and $_ = read_file("$blockpath/size", { err_mode => "quiet" }))? int(s/^\s+|\s+$//gr)*512: undef,
+			removable => (-f "$blockpath/removable" and $_ = read_file("$blockpath/removable", { err_mode => "quiet" }))? s/^\s+|\s+$//gr: undef,
+			partition => (-f "$blockpath/partition" and $_ = read_file("$blockpath/partition", { err_mode => "quiet" }))? s/^\s+|\s+$//gr: undef,
 			dmname => undef,
 			dmpath => undef,
 		};
-		if (-f "$blockpath/dm/name" and my $dmname = read_file("$blockpath/dm/name"))
+		if (-f "$blockpath/dm/name" and my $dmname = read_file("$blockpath/dm/name", { err_mode => "quiet" }))
 		{
 			chomp $dmname;
 			$disk->{dmname} = $dmname;
 			$disk->{dmpath} = "/dev/mapper/$dmname";
 		}
 		my $dmpath = $disk->{dmpath}? $disk->{dmpath}: "";
-		for my $mount (grep(/^(\Q$disk->{devpath}\E|\Q$dmpath\E)\s+/, (-f "/proc/mounts")? read_file("/proc/mounts"): ()))
+		for my $mount (grep(/^(\Q$disk->{devpath}\E|\Q$dmpath\E)\s+/, (-f "/proc/mounts")? read_file("/proc/mounts", { err_mode => "quiet" }): ()))
 		{
 			chomp $mount;
 			my ($devpath, $mountpoint, $fstype) = $mount =~ /^(\S+)\s+(\S+)\s+(\S+)\s+/;
@@ -121,7 +121,7 @@ sub stats
 	{
 		my $disk = $disks->{$devname};
 		next unless -f "$disk->{blockpath}/stat";
-		my $statLine = read_file("$disk->{blockpath}/stat");
+		my $statLine = read_file("$disk->{blockpath}/stat", { err_mode => "quiet" });
 		next unless $statLine;
 		chomp $statLine;
 		my $stat = { 'epoch' => time() };
@@ -151,18 +151,22 @@ sub analyzeStats
 	my $tmpPrefix = "/tmp/".__PACKAGE__ =~ s/\Q::\E/-/gr.".analyzeStats.";
 	for my $tmpPath (sort {$b cmp $a} glob("$tmpPrefix*"))
 	{
-		if (my ($epoch, $pid) = $tmpPath =~ /^\Q$tmpPrefix\E(\d*)\.(\d*)/) 
+		if (my ($epoch, $pid) = $tmpPath =~ /^\Q$tmpPrefix\E(\d*)\.(\d*)/)
 		{
-			my $tmp = read_file($tmpPath);
 			if ($now-$epoch < 1*60)
 			{
-				eval { $stats = from_json($tmp) } if not $stats and $tmp;
+				if (not $stats)
+				{
+					my $tmp = read_file($tmpPath, { err_mode => "quiet" });
+					eval { $stats = from_json($tmp) } if $tmp;
+				}
 				next;
 			}
-			if (not $oldStats and $tmp)
+			if (not $oldStats)
 			{
-				eval { $oldStats = from_json($tmp) };
-				next unless $@;
+				my $tmp = read_file($tmpPath, { err_mode => "quiet" });
+				eval { $oldStats = from_json($tmp) } if $tmp;
+				next unless not $tmp or $@;
 			}
 			unlink($tmpPath) if $now-$epoch > 2*60;
 			next;
@@ -172,7 +176,9 @@ sub analyzeStats
 	unless ($stats)
 	{
 		$stats = stats();
-		write_file("$tmpPrefix$now.$$", to_json($stats, {pretty => 1}));
+		my $tmp;
+		eval { $tmp = to_json($stats, {pretty => 1}) };
+		write_file("$tmpPrefix$now.$$", { err_mode => "quiet" }, $tmp) if $tmp;
 	}
 	return unless $oldStats;
 	my $result = {};
@@ -230,7 +236,7 @@ sub _bps
 	my $result = 0;
 	my $analyzed = analyzeStats();
 	my $status = $analyzed->{$devname} if $analyzed;
-	$result = $status->{"bps_$type"} if $status;
+	$result = sprintf("%.2f", $status->{"bps_$type"}) if $status;
 	print $result;
 	return $result;
 }
@@ -242,7 +248,7 @@ sub _iops
 	my $result = 0;
 	my $analyzed = analyzeStats();
 	my $status = $analyzed->{$devname} if $analyzed;
-	$result = $status->{"iops_$type"} if $status;
+	$result = sprintf("%.2f", $status->{"iops_$type"}) if $status;
 	print $result;
 	return $result;
 }
@@ -254,7 +260,7 @@ sub _ioutil
 	my $result = 0;
 	my $analyzed = analyzeStats();
 	my $status = $analyzed->{$devname} if $analyzed;
-	$result = $status->{"ioutil_$type"} if $status;
+	$result = sprintf("%.2f", $status->{"ioutil_$type"}) if $status;
 	print $result;
 	return $result;
 }
